// OwnerList.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "OwnerList.h"
#include "log.h"
#include "Trace.h"
// COwnerList

IMPLEMENT_DYNAMIC(COwnerList, CListBox)

COwnerList::COwnerList()
   {
    TraceCompareItem = FALSE;
    TraceMeasureItem = FALSE;
    TraceDrawItem = FALSE;
    TraceDeleteItem = TRUE;
   }

COwnerList::~COwnerList()
   {
   }


/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(COwnerList, CListBox)
END_MESSAGE_MAP()



// COwnerList message handlers


/****************************************************************************
*                              COwnerList::Log
* Inputs:
*       TraceEvent * e:
* Result: void
*       
* Effect: 
*       Logs a message
****************************************************************************/

void COwnerList::Log(TraceEvent * e)
    {
     GetParent()->SendMessage(UWM_LOG,(WPARAM)(e));
    } // COwnerList::Log

/****************************************************************************
*                           COwnerList::CompareItem
* Inputs:
*       LPCOMPAREITEMSTRUCT cis:
* Result: int
*       
* Effect: 
*       Compares two items of an owner-draw List
****************************************************************************/

int COwnerList::CompareItem(LPCOMPAREITEMSTRUCT cis)
    {
     /****************
        UINT        CtlType;
        UINT        CtlID;
        HWND        hwndItem;
        UINT        itemID1;
        ULONG_PTR   itemData1;
        UINT        itemID2;
        ULONG_PTR   itemData2;
        DWORD       dwLocaleId;
     ****************/
     if(TraceCompareItem)
        { /* trace it */
         Log(new TraceFormatComment(cis->CtlID, _T("COMPAREITEMSTRUCT")));
         Log(new TraceFormatComment(cis->CtlID, _T("    UINT CtlType %d"), cis->CtlType));
         Log(new TraceFormatComment(cis->CtlID, _T("    UINT CtlID %d"), cis->CtlID));
         Log(new TraceFormatComment(cis->CtlID, _T("    HWND hwndItem %p"), cis->hwndItem));
         Log(new TraceFormatComment(cis->CtlID, _T("    UINT itemID1 %d"), cis->itemID1));
         CString itemData1;
         if(cis->itemData1 == NULL)
            itemData1 = _T("NULL");
         else
            itemData1.Format(_T("%p \"%s\""), cis->itemData1, (LPCTSTR)cis->itemData1);
         Log(new TraceFormatComment(cis->CtlID, _T("    ULONG_PTR itemData1 %s"), itemData1));
         Log(new TraceFormatComment(cis->CtlID, _T("    UINT itemID2 %d"), cis->itemID2));

         CString itemData2;
         if(cis->itemData2 == NULL)
            itemData2 = _T("NULL");
         else
            itemData2.Format(_T("%p \"%s\""), cis->itemData2, (LPCTSTR)cis->itemData2);
         
         Log(new TraceFormatComment(cis->CtlID, _T("    ULONG_PTR itemData2 %s"), itemData2));
         Log(new TraceFormatComment(cis->CtlID, _T("    DWORD dwLocaleId 0x%08x"), cis->dwLocaleId));
        } /* trace it */
     int result = lstrcmp((LPCTSTR)cis->itemData1, (LPCTSTR)cis->itemData2);

     if(TraceCompareItem)
        Log(new TraceFormatComment(cis->CtlID, _T("    => %d"), result));

     return result;
    }


/****************************************************************************
*                           COwnerList::DeleteItem
* Inputs:
*       LPDELETEITEMSTRUCT dis: 
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void COwnerList::DeleteItem(LPDELETEITEMSTRUCT dis)
    {
     /*
      UINT       CtlType;
      UINT       CtlID;
      UINT       itemID;
      HWND       hwndItem;
      ULONG_PTR  itemData;
    */
     if(TraceDeleteItem)
        { /* trace it */
         Log(new TraceFormatComment(dis->itemID, _T("DELETEITEMSTRUCT")));
         Log(new TraceFormatComment(dis->itemID, _T("   UINT CtlType %d"), dis->CtlType));
         Log(new TraceFormatComment(dis->itemID, _T("   UINT CtlID %d"), dis->CtlID));
         Log(new TraceFormatComment(dis->itemID, _T("   UINT itemId %d"), dis->itemID));
         Log(new TraceFormatComment(dis->itemID, _T("   hwndItem %p"), dis->hwndItem));
         Log(new TraceFormatComment(dis->itemID, _T("   itemData %p \"%s\""), dis->itemData, dis->itemData));
        } /* trace it */
     // Note: In the test sample, the itemData is always a pointer to a static element and cannot be deleted
    }


/****************************************************************************
*                           COwnerList::MeasureItem
* Inputs:
*       LPMEASUREITEMSTRUCT mis
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void COwnerList::MeasureItem(LPMEASUREITEMSTRUCT mis)
    {
     /****************
        UINT       CtlType;
        UINT       CtlID;
        UINT       itemID;
        UINT       itemWidth;
        UINT       itemHeight;
        ULONG_PTR  itemData;
     ****************/
     if(TraceMeasureItem)
        { /* trace it */
         Log(new TraceFormatComment(mis->itemID, _T("MEASUREITEMSTRUCT")));
         Log(new TraceFormatComment(mis->itemID, _T("    UINT   CtlType %d"), mis->CtlType));
         Log(new TraceFormatComment(mis->itemID, _T("    UINT   CtlID %d"), mis->CtlID));
         Log(new TraceFormatComment(mis->itemID, _T("    UINT   itemID %d"), mis->itemID));
         Log(new TraceFormatComment(mis->itemID, _T("    UINT   itemWidth %d"), mis->itemWidth));
         Log(new TraceFormatComment(mis->itemID, _T("    UINT   itemHeight %d"), mis->itemHeight));
         CString itemData;
         if(mis->itemData == 0)
            itemData = _T("NULL");
         else
            itemData.Format(_T("%p \"%s\""), mis->itemData, (LPCTSTR)mis->itemData);
         Log(new TraceFormatComment(mis->itemID, _T("    ULONG_PTR, itemData %s"), itemData));
        } /* trace it */
    }


/****************************************************************************
*                            COwnerList::DrawItem
* Inputs:
*       LPDRAWITEMSTRUCT dis
* Result: void
*       
* Effect: 
*       
****************************************************************************/

void COwnerList::DrawItem(LPDRAWITEMSTRUCT dis)
    {
     if(TraceDrawItem)
        { /* trace DRAWITEM */
         /*****************
            UINT        CtlType;
            UINT        CtlID;
            UINT        itemID;
            UINT        itemAction;
            UINT        itemState;
            HWND        hwndItem;
            HDC         hDC;
            RECT        rcItem;
            ULONG_PTR   itemData;
         *****************/

         CString sep;
#define AddBit(x, s)                                \
                  if( (dis->itemAction & (x)) != 0) \
                      {                             \
                       s += sep;                    \
                       s += _T(#x);                 \
                       sep = _T(" | ");             \
                      }


         Log(new TraceComment(dis->itemID, _T("DRAWITEMSTRUCT")));
         Log(new TraceFormatComment(dis->itemID, _T("    UINT      CtlType    %d"), dis->CtlType));
         Log(new TraceFormatComment(dis->itemID, _T("    UINT      CtlId      %d"), dis->CtlID));
         Log(new TraceFormatComment(dis->itemID, _T("    UINT      itemId     %d"), dis->itemID));
         CString itemAction;
         sep.Empty();
         AddBit(ODA_DRAWENTIRE, itemAction);
         AddBit(ODA_SELECT,     itemAction);
         AddBit(ODA_FOCUS,      itemAction);

         Log(new TraceFormatComment(dis->itemID, _T("    UINT      itemAction 0x%08x %s"), dis->itemAction, itemAction));
         CString itemState;
         sep.Empty();
         AddBit(ODS_SELECTED,     itemState);
         AddBit(ODS_GRAYED,       itemState);
         AddBit(ODS_DISABLED,     itemState);
         AddBit(ODS_CHECKED,      itemState);
         AddBit(ODS_DEFAULT,      itemState);
         AddBit(ODS_COMBOBOXEDIT, itemState);
         AddBit(ODS_HOTLIGHT,     itemState);
         AddBit(ODS_INACTIVE,     itemState);
         AddBit(ODS_NOACCEL,      itemState);
         AddBit(ODS_NOFOCUSRECT,  itemState);

         Log(new TraceFormatComment(dis->itemID, _T("    UINT      itemState  0x%08x, %s"), dis->itemState, itemState));
         Log(new TraceFormatComment(dis->itemID, _T("    HWND      hWndItem   %p"), dis->hwndItem));
         Log(new TraceFormatComment(dis->itemID, _T("    HDC       hDC        %p"), dis->hDC));
         Log(new TraceFormatComment(dis->itemID, _T("    RECT      rcItem     [L %d, T %d, R %d, B %d]"),
                                                dis->rcItem.left, dis->rcItem.top, dis->rcItem.right, dis->rcItem.bottom));
         CString itemData;
         if(dis->itemData == 0)
            itemData.Format(_T("NULL"));
         else
            if(dis->itemData == ((ULONG_PTR)-1))
               itemData.Format(_T("%p"), dis->itemData);
            else
               itemData.Format(_T("%p, \"%s\""), dis->itemData, (LPCTSTR)dis->itemData);

         Log(new TraceFormatComment(dis->itemID, _T("    ULONG_PTR itemData   %s"), itemData));
        } /* trace DRAWITEM */

     CRect r = dis->rcItem;
     CDC * dc = CDC::FromHandle(dis->hDC);
     int save = dc->SaveDC();

     if(dis->itemID == (UINT)-1)
        { /* nothing present */
         if(dis->itemAction & ODA_FOCUS)
            dc->DrawFocusRect(&r);
         return;
        } /* nothing present */

     COLORREF tx;
     COLORREF bk;
     if(dis->itemState & ODS_SELECTED)
        { /* selected */
         tx = ::GetSysColor(COLOR_WINDOW);
         bk = (::GetFocus() == m_hWnd) != 0 ? ::GetSysColor(COLOR_HIGHLIGHT) : ::GetSysColor(COLOR_GRAYTEXT);
        } /* selected */
     else
        { /* not selected */
         tx = IsWindowEnabled() ? ::GetSysColor(COLOR_WINDOWTEXT) : ::GetSysColor(COLOR_GRAYTEXT);
         bk = ::GetSysColor(COLOR_WINDOW);
        } /* not selected */

     CString s;

     dc->SetBkMode(TRANSPARENT);
     dc->SetTextColor(tx);
     dc->FillSolidRect(&r, bk);

     DWORD style = GetStyle();
     if(style & LBS_HASSTRINGS)
        GetText(dis->itemID, s);
     else
        s = CString((LPCTSTR)(dis->itemData));

     dc->TextOut(dis->rcItem.left, dis->rcItem.top, s);

     if(dis->itemState & ODS_FOCUS)
        dc->DrawFocusRect(&r);

     dc->RestoreDC(save);
    }
